from dila import data


def add_resource(resource_name):
    data.add_resource(resource_name)


def get_resources():
    return list(data.get_resources())


def get_resource(pk):
    return data.get_resource(pk)



"""Add resource name

Revision ID: 04d870437e14
Revises: f8927e919732
Create Date: 2017-04-27 12:31:55.646804

"""

# revision identifiers, used by Alembic.
revision = '04d870437e14'
down_revision = 'f8927e919732'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('resource', sa.Column('name', sa.Text(), nullable=False))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('resource', 'name')
    # ### end Alembic commands ###



import collections

TranslatedStringData = collections.namedtuple(
    'TranslatedString',
    ['pk', 'base_string', 'translation', 'comment', 'translator_comment', 'context']
)

Resource = collections.namedtuple(
    'Resource',
    ['pk', 'name']
)



import sqlalchemy
import sqlalchemy.dialects.postgresql as postgres_dialect
from sqlalchemy import orm

from dila.application import structures
from dila.data import engine
from dila.data import resources


class TranslatedString(engine.Base):
    __tablename__ = 'translated_string'
    id = sqlalchemy.Column(postgres_dialect.UUID(as_uuid=True),
                           server_default=sqlalchemy.text("uuid_generate_v4()"), primary_key=True,
                           nullable=False)
    resource_pk = sqlalchemy.Column(
        postgres_dialect.UUID(as_uuid=True), sqlalchemy.ForeignKey(resources.Resource.id), nullable=False)
    resources = orm.relationship(resources.Resource, backref=orm.backref('translated_strings'))
    base_string = sqlalchemy.Column(sqlalchemy.Text, nullable=False)
    translation = sqlalchemy.Column(sqlalchemy.Text, nullable=False, default='')
    comment = sqlalchemy.Column(sqlalchemy.Text, nullable=False, default='')
    translator_comment = sqlalchemy.Column(sqlalchemy.Text, nullable=False, default='')
    context = sqlalchemy.Column(sqlalchemy.Text, nullable=False, default='')

    def as_data(self):
        return structures.TranslatedStringData(
            self.id,
            self.base_string,
            self.translation,
            self.comment,
            self.translator_comment,
            self.context,
        )


def add_translated_string(resource_pk, base_string, *, translation, comment, translator_comment, context):
    engine.session.add(TranslatedString(
        resource_pk=resource_pk, base_string=base_string, translation=translation, comment=comment,
        translator_comment=translator_comment, context=context
    ))
    engine.session.flush()


def get_translated_strings(resource_pk):
    for translated_string in TranslatedString.query.filter_by(resource_pk=resource_pk):
        yield translated_string.as_data()


def get_translated_string(pk):
    return TranslatedString.query.get(pk).as_data()


def set_translated_string(pk, **kwargs):
    update = {
        getattr(TranslatedString, key): value for key, value in kwargs.items()
    }
    TranslatedString.query.filter(TranslatedString.id == pk).update(update)



import flask

from dila import application
from dila import config
from dila.frontend import initialize
from dila.frontend.flask import views


def main(initialized=False):
    if not initialized:
        initialize.initialize()
    app = create_app()

    prepare_application(app)
    return app


def prepare_application(app):
    @app.teardown_appcontext
    def shutdown_session(exception=None):
        application.shutdown_session(exception=exception)


def create_app():
    app = flask.Flask(__name__)
    app.config.from_mapping(
        SERVER_NAME=config.SERVER_NAME,
        SECRET_KEY=config.SECRET_KEY,
        TESTING=config.ENV == 'test',
        DEBUG=config.DEBUG,
    )
    app.register_blueprint(views.blueprint)
    return app



import polib

from dila import data


def get_po_file(resource_pk):
    po = polib.POFile()
    for string in data.get_translated_strings(resource_pk):
        po.append(polib.POEntry(
            msgid=string.base_string,
            msgstr=string.translation,
            msgctxt=string.context,
            comment=string.comment,
            tcomment=string.translator_comment,
        ))
    return str(po)


def upload_translated_po_file(resource_pk, content):
    po = polib.pofile(content)
    for entry in po:
        if not entry.obsolete:
            data.add_translated_string(
                resource_pk,
                entry.msgid,
                context=entry.msgctxt,
                translation=entry.msgstr,
                comment=entry.comment,
                translator_comment=entry.tcomment,
            )


def get_translated_strings(resource_pk):
    return data.get_translated_strings(resource_pk)


def get_translated_string(resource_pk, pk):
    return data.get_translated_string(pk)


def set_translated_string(resource_pk, pk, **kwargs):
    data.set_translated_string(pk, **kwargs)



import flask_wtf
import wtforms


class PoFileUpload(flask_wtf.FlaskForm):
    po_file = wtforms.FileField('po file')


class TranslationForm(flask_wtf.FlaskForm):
    translation = wtforms.StringField()


class NewResourceForm(flask_wtf.FlaskForm):
    new_resource_name = wtforms.StringField('New resource')



"""Add translated_strings

Revision ID: 0ab02337f372
Revises:
Create Date: 2017-03-08 14:50:37.058424

"""

# revision identifiers, used by Alembic.
revision = '0ab02337f372'
down_revision = None
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.execute('CREATE EXTENSION IF NOT EXISTS "uuid-ossp";')
    op.create_table('translated_string',
    sa.Column('id', postgresql.UUID(as_uuid=True), server_default=sa.text('uuid_generate_v4()'), nullable=False),
    sa.Column('base_string', sa.Text(), nullable=True),
    sa.Column('translation', sa.Text(), nullable=True),
    sa.Column('comment', sa.Text(), nullable=True),
    sa.Column('translator_comment', sa.Text(), nullable=True),
    sa.Column('context', sa.Text(), nullable=True),
    sa.PrimaryKeyConstraint('id')
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('translated_string')
    # ### end Alembic commands ###



import uuid

import sqlalchemy
import sqlalchemy.dialects.postgresql as postgres_dialect

from dila.application import structures
from dila.data import engine


class Resource(engine.Base):
    __tablename__ = 'resource'
    id = sqlalchemy.Column(postgres_dialect.UUID(as_uuid=True), default=uuid.uuid4,
                           server_default=sqlalchemy.text("uuid_generate_v4()"), primary_key=True,
                           nullable=False)
    name = sqlalchemy.Column(sqlalchemy.Text(), nullable=False)

    def as_data(self):
        return structures.Resource(pk=self.id, name=self.name)


def add_resource(name):
    resource = Resource(name=name)
    engine.session.add(resource)
    engine.session.flush()
    return resource.as_data()


def get_resource(pk):
    return Resource.query.get(pk).as_data()


def get_resources():
    for resource in Resource.query.all():
        yield resource.as_data()



import re

SERVER_NAME = None
SECRET_KEY = None
ENV = 'production'
DEBUG = False
DATABASE_URL = ''


def setup_from_module(module_name):
    module = __import__(module_name)
    setting_re = re.compile("^[A-Z0-9_]*$")
    setup(**{key: value for key, value in module.__dict__.items() if setting_re.match(key)})


def setup(**mapping):
    globals().update(mapping)



from .meta import setup
from .meta import shutdown_session
from .resources import add_resource
from .resources import get_resource
from .resources import get_resources
from .translations import get_po_file
from .translations import get_translated_string
from .translations import get_translated_strings
from .translations import set_translated_string
from .translations import upload_translated_po_file

__all__ = [setup, shutdown_session, upload_translated_po_file, get_translated_string, get_translated_strings,
           set_translated_string, get_po_file, add_resource, get_resource, get_resources]



from __future__ import with_statement
from alembic import context
from logging.config import fileConfig
import sqlalchemy.pool
import sqlalchemy.engine

from dila.data import engine
from dila import config as dila_config

from dila.frontend import initialize

__import__('dila.data.translated_strings')
initialize.initialize()

# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
config = context.config

# Interpret the config file for Python logging.
# This line sets up loggers basically.
fileConfig(config.config_file_name)

# add your model's MetaData object here
# for 'autogenerate' support
# from myapp import mymodel
# target_metadata = mymodel.Base.metadata
target_metadata = engine.Base.metadata


def run_migrations_offline():
    context.configure(
        url=dila_config.DATABASE_URL, target_metadata=target_metadata, literal_binds=True)

    with context.begin_transaction():
        context.run_migrations()


def run_migrations_online():
    connectable = sqlalchemy.engine.create_engine(
        dila_config.DATABASE_URL,
        poolclass=sqlalchemy.pool.NullPool
    )
    with connectable.connect() as connection:
        context.configure(
            connection=connection,
            target_metadata=target_metadata
        )

        with context.begin_transaction():
            context.run_migrations()

if context.is_offline_mode():
    run_migrations_offline()
else:
    run_migrations_online()



import sqlalchemy.ext.declarative
import sqlalchemy.orm

from sqlalchemy_postgres_autocommit import databases
from dila import config


database = databases.Database()
session = sqlalchemy.orm.scoped_session(database.Session)
Base = sqlalchemy.ext.declarative.declarative_base()
Base.query = session.query_property()


def setup():
    database.connect(config.DATABASE_URL)


def shutdown_session(exception=None):
    session.remove()



import os
from dila import application
from dila import config


def initialize():
    initialize_config()
    application.setup()


def initialize_config():
    if 'DILA_CONFIG_MODULE' in os.environ:
        module_name = os.environ['DILA_CONFIG_MODULE']
        config.setup_from_module(module_name)



import signal

import cmd2
import sys

import os

import pathlib

from dila.frontend import flask
from dila.frontend import initialize

TERMINATION_SIGNALS = [signal.SIGTERM, signal.SIGQUIT, signal.SIGINT]


def register_termination_signals():
    def terminate(signum, frame):
        sys.exit(0)

    for signum in TERMINATION_SIGNALS:
        signal.signal(signum, terminate)

class Dila(cmd2.Cmd):
    """Dila Commandline."""
    def do_run_dev_server(self, arg):
        print('Running dev server')
        flask.main(initialized=True).run(host='0.0.0.0', port=80)

    def do_migrate(self, arg):
        print('migrating')
        os.execv('/usr/local/bin/alembic', [
            'alembic', '-c', str(pathlib.Path(__file__).parent.parent / 'alembic.ini'),
            'upgrade', 'head'])


def run():
    import sys
    register_termination_signals()
    initialize.initialize()
    if len(sys.argv) > 1:
        Dila().onecmd(' '.join(sys.argv[1:]))
    else:
        Dila().cmdloop()


if __name__ == '__main__':
    run()



from dila.data.translated_strings import add_translated_string
from dila.data.translated_strings import get_translated_string
from dila.data.translated_strings import get_translated_strings
from dila.data.translated_strings import set_translated_string
from dila.data.resources import add_resource
from dila.data.resources import get_resource
from dila.data.resources import get_resources
from dila.data.engine import setup
from dila.data.engine import shutdown_session

__all__ = [
    add_translated_string, get_translated_string, get_translated_strings, set_translated_string, add_resource,
    get_resource, get_resources, setup, shutdown_session
]



from pip.req import parse_requirements
from setuptools import find_packages
from setuptools import setup


setup(
    name='dila',
    version='0.0.5',
    description='Dila is a open source web-based translation platform for translators, content creators and developers.',
    author='Jakub Skiepko',
    author_email='it@socialwifi.com',
    url='https://github.com/socialwifi/dila',
    packages=find_packages(exclude=['tests']),
    install_requires=[str(ir.req) for ir in parse_requirements('base_requirements.txt', session=False)],
    entry_points={
        'console_scripts': [
            'dila = dila.frontend.cli:run',
        ],
    },
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    license='BSD',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ],
    package_dir={'sw_cli': 'sw_cli'},
    package_data={'sw_cli': ['alembic.ini']},
    include_package_data=True,
)



"""Set translations not null

Revision ID: 58e7f2fc7ff7
Revises: 0ab02337f372
Create Date: 2017-03-09 10:23:37.343117

"""

# revision identifiers, used by Alembic.
revision = '58e7f2fc7ff7'
down_revision = '0ab02337f372'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.execute("UPDATE translated_string SET base_string='' WHERE base_string is NULL")
    op.alter_column('translated_string', 'base_string',
               existing_type=sa.TEXT(),
               nullable=False)
    op.execute("UPDATE translated_string SET comment='' WHERE comment is NULL")
    op.alter_column('translated_string', 'comment',
               existing_type=sa.TEXT(),
               nullable=False)
    op.execute("UPDATE translated_string SET context='' WHERE context is NULL")
    op.alter_column('translated_string', 'context',
               existing_type=sa.TEXT(),
               nullable=False)
    op.execute("UPDATE translated_string SET translation='' WHERE translation is NULL")
    op.alter_column('translated_string', 'translation',
               existing_type=sa.TEXT(),
               nullable=False)
    op.execute("UPDATE translated_string SET translator_comment='' WHERE translator_comment is NULL")
    op.alter_column('translated_string', 'translator_comment',
               existing_type=sa.TEXT(),
               nullable=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column('translated_string', 'translator_comment',
               existing_type=sa.TEXT(),
               nullable=True)
    op.alter_column('translated_string', 'translation',
               existing_type=sa.TEXT(),
               nullable=True)
    op.alter_column('translated_string', 'context',
               existing_type=sa.TEXT(),
               nullable=True)
    op.alter_column('translated_string', 'comment',
               existing_type=sa.TEXT(),
               nullable=True)
    op.alter_column('translated_string', 'base_string',
               existing_type=sa.TEXT(),
               nullable=True)
    # ### end Alembic commands ###



"""Add resource model.

Revision ID: f8927e919732
Revises: 58e7f2fc7ff7
Create Date: 2017-04-27 10:07:33.766861

"""

# revision identifiers, used by Alembic.
revision = 'f8927e919732'
down_revision = '58e7f2fc7ff7'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql


def upgrade():
    op.create_table('resource',
    sa.Column('id', postgresql.UUID(as_uuid=True), server_default=sa.text('uuid_generate_v4()'), nullable=False),
    sa.PrimaryKeyConstraint('id')
    )
    op.add_column('translated_string', sa.Column('resource_pk', postgresql.UUID(as_uuid=True), nullable=False))
    op.create_foreign_key(None, 'translated_string', 'resource', ['resource_pk'], ['id'])


def downgrade():
    op.drop_constraint(None, 'translated_string', type_='foreignkey')
    op.drop_column('translated_string', 'resource_pk')
    op.drop_table('resource')



import flask
from cached_property import cached_property
from flask import views

from dila import application
from dila.frontend.flask import forms

blueprint = flask.Blueprint('main', __name__)


class HomeView(views.MethodView):
    def get(self):
        return flask.render_template('home.html', **self.context)

    def post(self):
        if self.form.validate():
            application.add_resource(self.form.new_resource_name.data)
            flask.flash('Resource created')
            return flask.redirect(flask.url_for('main.home'))
        else:
            return self.get()

    @property
    def context(self):
        return {
            'resources': self.resources,
            'form': self.form
        }

    @cached_property
    def resources(self):
        return application.get_resources()

    @cached_property
    def form(self):
        return forms.NewResourceForm()


blueprint.add_url_rule('/', view_func=HomeView.as_view('home'))


class ResourceView(views.MethodView):
    def dispatch_request(self, *args, resource_pk):
        self.resource_pk = resource_pk
        return super().dispatch_request(*args)

    def get(self):
        return flask.render_template('resource.html', **self.context)

    def post(self):
        if self.form.validate():
            po_content = flask.request.files[self.form.po_file.name].read().decode()
            application.upload_translated_po_file(self.resource_pk, po_content)
            flask.flash('File uploaded')
            return flask.redirect(flask.url_for('main.resource', resource_pk=self.resource_pk))
        else:
            return self.get()

    @property
    def context(self):
        return {
            'translated_strings': self.translated_strings,
            'form': self.form,
            'resource_pk': self.resource_pk,
        }

    @cached_property
    def form(self):
        return forms.PoFileUpload()

    @cached_property
    def translated_strings(self):
        return application.get_translated_strings(self.resource_pk)

blueprint.add_url_rule('/<resource_pk>/', view_func=ResourceView.as_view('resource'))


class TranslatedStringEditor(views.MethodView):
    def dispatch_request(self, *args, resource_pk, pk):
        self.pk = pk
        self.resource_pk = resource_pk
        return super().dispatch_request(*args)

    def get(self):
        return flask.render_template('translated_string.html', **self.context)

    def post(self):
        if self.form.validate():
            application.set_translated_string(self.resource_pk, self.pk, translation=self.form.data['translation'])
            flask.flash('Translation changed')
            return flask.redirect(flask.url_for('main.resource', resource_pk=self.resource_pk))
        else:
            return self.get()

    @property
    def context(self):
        return {
            'form': self.form,
            'translated_string': self.translated_string,
            'resource_pk': self.resource_pk,
        }

    @cached_property
    def form(self):
        return forms.TranslationForm(obj=self.translated_string)

    @cached_property
    def translated_string(self):
        return application.get_translated_string(self.resource_pk, self.pk)


blueprint.add_url_rule('/<resource_pk>/edit/<pk>/', view_func=TranslatedStringEditor.as_view('translated_string'))


class PoFileDownload(views.MethodView):
    def get(self, *args, resource_pk):
        response = flask.make_response(application.get_po_file(resource_pk))
        response.headers["Content-Disposition"] = "attachment; filename=translations.po"
        return response


blueprint.add_url_rule('/<resource_pk>/po-file/', view_func=PoFileDownload.as_view('po_file_download'))
